{Written by Attila Szomor, his email: aszomor@pointer.hu}
unit HCForms;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, HCLists;

const
  CaptionMinHeightPattern= 10;
  CaptionHeightThreshold= 20;

  clHCDarkGreen= {clBlack}  $00308912;
  clHCMidGreen= {clTeal  }  $0089CEB2;
  clHCLightGreen= {clYellow } $00CCF7E6;

type
  THCForm = class(TForm)
  private
    { Private declarations }
    fRGN: hRGN;
    fCaptionHeight: integer;  {height of the title bar in pixels}
    fActiveCaption: boolean; {state of the title bar}
    fSysMenu, fMin, fMax, fClose: boolean;
    fMenuUp: boolean;        {state of the system menu}
    fSysMenuItems: TSortedDWORDKeyList;
    fSysMenuWidth: integer;
    function  TestWinStyle(dwStyleBit : longInt): boolean;
    {function  TestWinStyleEx(dwExStyleBit: longInt): boolean;}
    function  HasCaption: boolean;
    function  GetCaptionRect(var Rect: TRect): boolean;
    function  GetCaptionButtonRect(Which: word; var Rect: TRect): boolean;
    procedure DrawCaptionButton(DC: HDC; Which: word; Pressed: boolean);
    function  DrawCaption: boolean;
    function  DepressCaptionButton(Which: word) : boolean;
    function  DoSysMenu : boolean;
  protected
    procedure WndProc(var Message: TMessage); override;
    procedure WMNCCreate(var Message: TWMNCCreate); message WM_NCCREATE;
    procedure WMNCCalcSize(var Message: TWMNCCalcSize); message WM_NCCALCSIZE;
    procedure WMNCHitTest(var Message: TWMNCHitTest); message WM_NCHITTEST;
    procedure WMNCLButtonDblClk(var Message: TWMNCLButtonDblClk); message WM_NCLBUTTONDBLCLK;
    procedure WMNCLButtonDown(var Message: TWMNCLButtonDown); message WM_NCLBUTTONDOWN;

    procedure WMSize(var Message: TWMSize); message WM_SIZE;
    procedure WMSysChar(var Message: TWMSysChar); message WM_SYSCHAR;
    procedure WMCommand(var Message: TMessage); message WM_COMMAND;
    procedure WMKeyDown(var Message: TWMKeyDown); message WM_KEYDOWN;
    procedure WMKeyUp(var Message: TWMKeyUp); message WM_KEYUP;
    procedure WMSysKeyDown(var Message: TWMSysKeyDown); message WM_SYSKEYDOWN;
    procedure WMSysKeyUp(var Message: TWMSysKeyUp); message WM_SYSKEYUP;
    procedure WMGetMinMaxInfo(var Message: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
  public
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    { Public declarations }
  end;

implementation

Uses Windows;

{$R *.DFM}

type
  PSysMenuItem= ^TSysMenuItem;
  TSysMenuItem= Packed Record
    SysMenuItemID: DWORD;
    SysMenuItemText: PChar;
  End;

(* !!!! HTHELP -->> WS_EX_CONTEXTHELP  !!!! *)

function THCForm.TestWinStyle(dwStyleBit: longInt): boolean;
begin
  Result:= ((GetWindowLong(Handle, GWL_STYLE) AND dwStyleBit) <> 0);
end;
{
function THCForm.TestWinStyleEx(dwExStyleBit: longInt): boolean;
begin
  Result:= ((GetWindowLong(Handle, GWL_EXSTYLE) AND dwExStyleBit) <> 0);
end;
}
function THCForm.HasCaption: boolean;
begin
  Result:= TestWinStyle(WS_CAPTION) {BorderStyle <> bsNone};
end;

function THCForm.GetCaptionRect(var Rect: TRect): boolean;
begin
  If HasCaption Then Begin
    GetWindowRect(Handle, Rect);
    {Adjust for borders}
    If TestWinStyle(WS_THICKFRAME) Then Begin
      InflateRect(Rect, -GetSystemMetrics(SM_CXFRAME),-GetSystemMetrics(SM_CYFRAME));
    End Else Begin
      If TestWinStyle(DS_MODALFRAME) Then Begin
        InflateRect(Rect,
          -(GetSystemMetrics(SM_CXDLGFRAME)+ GetSystemMetrics(SM_CXBORDER)),
          -(GetSystemMetrics(SM_CYDLGFRAME)+ GetSystemMetrics(SM_CYBORDER))
        );
      End Else Begin
        If TestWinStyle(WS_BORDER) Then Begin
          InflateRect(Rect, -GetSystemMetrics(SM_CXBORDER),-GetSystemMetrics(SM_CYBORDER));
        End;
      End;
    End;
    Rect.Bottom:= Rect.Top + fCaptionHeight;
    Result:= True;
  End Else Begin
    SetRectEmpty(Rect);
    Result:= False;
  End;
end;

function THCForm.GetCaptionButtonRect(Which:word; var Rect: TRect): boolean;

  Procedure SetButtonRect;
  Var
    pButton: byte;
  Begin
    pButton:=0;
    Case Which Of
      HTMINBUTTON: Begin
        If TestWinStyle(WS_MAXIMIZEBOX) Then Begin
          Inc(pButton);
        End;
        If TestWinStyle(WS_SYSMENU) Then Begin
          Inc(pButton);
        End;
      End;
      HTMAXBUTTON: Begin
        If TestWinStyle(WS_SYSMENU) Then Begin
          Inc(pButton);
        End;
      End;
    End;
    { Draw them into the right side on caption /width=height/ }
    Dec(Rect.Right, fCaptionHeight * pButton + 1);
    Rect.Left:= Rect.Right-fCaptionHeight + 2;
  End;

begin
  Result:= False;
  If GetCaptionRect(Rect) Then Begin
    Inc(Rect.Top);
    Dec(Rect.Bottom);
    Case Which Of
      HTSYSMENU: Begin
        If TestWinStyle(WS_SYSMENU) Then Begin
          { Draw it into the left side on caption /width=height/ }
          Rect.Right:=Rect.Left + fCaptionHeight - 1;
          Result:= True;
        End;
      End;
      HTMINBUTTON: Begin
        If TestWinStyle(WS_MINIMIZEBOX) Then Begin
          SetButtonRect;
          Result:= True;
        End;
      End;
      HTMAXBUTTON: Begin
        If TestWinStyle(WS_MAXIMIZEBOX) Then Begin
          SetButtonRect;
          Result:= True;
        End;
      End;
      HTCLOSE: Begin
        If TestWinStyle(WS_SYSMENU) Then Begin
          SetButtonRect;
          Result:= True;
        End;
      End;
    End;
  End;
  If NOT Result Then SetRectEmpty(Rect);
end;

procedure THCForm.DrawCaptionButton(DC: HDC; Which: word; Pressed: boolean);
var
  Rect,RectBox: TRect;
  DC_Created: boolean;
  Pen: HPen;
begin
  {Get size & position of button, and convert to window coordinates}
  If GetCaptionButtonRect( Which, RectBox ) Then Begin
    If DC = 0 Then Begin
      DC_Created:= True;
      DC:= GetWindowDC(Handle);
    End Else DC_Created:= False;
    If DC <> 0 Then Begin
      Pen:= CreatePen(PS_SOLID,1,ColorToRGB(clBtnHighlight));

      GetWindowRect(Handle, Rect);
      OffsetRect(RectBox, -Rect.Left, -Rect.Top);

      {Calculate separator line to left of button !!!SysMenu->right!!!}
      Rect:= RectBox;
      If Which=HTSYSMENU Then
        Rect.Left:= Rect.Right
      Else Rect.Right:= Rect.Left;
      InflateRect(Rect,-1,-1);
      SetBkColor(DC, ColorToRGB(clHCMidGreen));
      ExtTextOut(DC, 0, 0, ETO_OPAQUE, @Rect, NIL, 0, NIL);

      {Fill button area}
      InflateRect(RectBox,-1,-1);
      If Pressed Then Begin
        SetBkColor(DC, ColorToRGB(clHCMidGreen))
      End Else Begin
        If fActiveCaption Then
          SetBkColor(DC, ColorToRGB(clHCDarkGreen))
        Else SetBkColor(DC, ColorToRGB(clHCLightGreen));
      End;
      ExtTextOut(DC, 0, 0, ETO_OPAQUE, @RectBox, NIL, 0, NIL);

      { Draw pattern of buttons }
      If fCaptionHeight > CaptionMinHeightPattern Then Begin
        Rect:= RectBox;
        Case Which Of
          HTSYSMENU: Begin
            RectBox.Top:= RectBox.Top + ((fCaptionHeight - 1) DIV 2);
            RectBox.Bottom:= RectBox.Top + 3;
            RectBox.Left:= RectBox.left + 4;
            RectBox.Right:=RectBox.Right - 3;
            SetBkColor(DC, ColorToRGB(clGray));
            ExtTextOut(DC, 0, 0, ETO_OPAQUE, @RectBox, NIL, 0, NIL);
            OffsetRect(RectBox, -1, -1);
            SetBkColor(DC, ColorToRGB(clBlack));
            ExtTextOut(DC, 0, 0, ETO_OPAQUE, @RectBox, NIL, 0, NIL);
            InflateRect(RectBox, -1, -1);
            SetBkColor(DC, ColorToRGB(clWhite));
            ExtTextOut(DC, 0, 0, ETO_OPAQUE, @RectBox, NIL, 0, NIL);
          End;
          HTMINBUTTON: Begin
            RectBox.Top:= RectBox.Top + (((fCaptionHeight - 1) DIV 4));
            RectBox.Left:= RectBox.Left + (((fCaptionHeight - 1) DIV 4));
            SelectObject(DC,Pen);
            MoveToEx(DC, RectBox.Left + 3, RectBox.Top + 3,NIL);
            LineTo(DC, RectBox.Left + 7, RectBox.Top + 3);
            MoveToEx(DC, RectBox.Left + 3, RectBox.Top + 3,NIL);
            LineTo(DC, RectBox.Left + 3, RectBox.Top + 7);
            SelectObject(DC,GetStockObject(BLACK_PEN));
            MoveToEx(DC, RectBox.Left + 7, RectBox.Top + 3,NIL);
            LineTo(DC, RectBox.Left + 7, RectBox.Top + 7);
            LineTo(DC, RectBox.Left + 3, RectBox.Top + 7);
          End;
          HTMAXBUTTON: Begin
            RectBox.Top:= RectBox.Top + (((fCaptionHeight - 1) DIV 4));
            RectBox.Left:= RectBox.Left + (((fCaptionHeight - 1) DIV 4));
            SelectObject(DC,Pen);
            MoveToEx(DC, RectBox.Left + 2, RectBox.Top + 1,NIL);
            LineTo(DC, RectBox.Left + 10, RectBox.Top + 1);
            MoveToEx(DC, RectBox.Left + 2, RectBox.Top + 1,NIL);
            LineTo(DC, RectBox.Left + 2, RectBox.Top + 10);
            SelectObject(DC,GetStockObject(BLACK_PEN));
            MoveToEx(DC, RectBox.Left + 10, RectBox.Top + 1,NIL);
            LineTo(DC, RectBox.Left + 10, RectBox.Top + 10);
            LineTo(DC, RectBox.Left + 2, RectBox.Top + 10);
          End;
          HTCLOSE: Begin
            SelectObject(DC,GetStockObject(BLACK_PEN));
            MoveToEx(DC, RectBox.Left, RectBox.Top-1,NIL);
            LineTo(DC, RectBox.Right-1, RectBox.Bottom);
            SelectObject(DC,Pen);
            MoveToEx(DC, RectBox.Left+1, RectBox.Top,NIL);
            LineTo(DC, RectBox.Right, RectBox.Bottom-1);

            MoveToEx(DC, RectBox.Right-1, RectBox.Top,NIL);
            LineTo(DC, RectBox.Left, RectBox.Bottom-1);
            SelectObject(DC,GetStockObject(BLACK_PEN));
            MoveToEx(DC, RectBox.Right, RectBox.Top-1,NIL);
            LineTo(DC, RectBox.Left+1, RectBox.Bottom);
          End;
        End;
      End;
      DeleteObject(Pen);
    End;
    If DC_Created Then Begin
      ReleaseDC(Handle, DC);
    End;
  End;
end;

function THCForm.DrawCaption: boolean;
var
  DC: hDC;
  Rect: TRect;
  rcCap: TRect;
  rgbText: TColor;
  rgbBkGrnd: TColor;
  hBrCap: hBrush;
  Pen: hPen;
  lpStr: PChar;
  TextLen: word;
  lFont: TLogFont;
  Font: HFont;
  Size: TSize;
  cX, cY: integer;
begin
  Result:=False;
  DC:= GetWindowDC(Handle);
  If DC <> 0 Then Begin
    {Determine colors}
    If fActiveCaption Then Begin
      rgbText:= ColorToRGB(clHCLightGreen);
      rgbBkGrnd:= ColorToRGB(clHCDarkGreen);
    End Else Begin
      rgbText:= ColorToRGB(clHCDarkGreen);
      rgbBkGrnd:= ColorToRGB(clHCLightGreen);
    End;
    SelectObject(DC, GetStockObject(NULL_PEN));
    hBrCap:= CreateSolidBrush(ColorToRGB(color));
    SelectObject(DC, hBrCap);
    {Calculate Caption rectangle in window coords.}
    GetCaptionRect(rcCap);
    GetWindowRect(Handle, Rect);
    OffsetRect(rcCap, -Rect.Left, -Rect.Top);

    {Calculate horizontal separator line below Caption}
    SetRect(Rect, rcCap.Left, rcCap.Top, rcCap.Right, rcCap.Bottom);
    hBrCap:= CreateSolidBrush(rgbBkGrnd);
    hBrCap:= SelectObject(DC, hBrCap);  {swap brushes with device context }
    Rectangle(DC, rcCap.Left, rcCap.Top, rcCap.Right , rcCap.Bottom);
    {Draw separator line}
    SetBkMode(DC, TRANSPARENT);
    SelectObject(DC, GetStockObject(NULL_BRUSH));
    SelectObject(DC, GetStockObject(NULL_PEN));
    SetBkColor(DC, rgbBkGrnd);
    ExtTextOut(DC, 0, 0, ETO_OPAQUE, @Rect, NIL, 0, NIL);

    {$IfDef HC_CAPTION_TEXT_HIGHLIGHT}
      {Shrink caption area to avoid overlapping control box & min/max buttons}
      If fSysMenu Then Inc(rcCap.Left, fCaptionHeight);
      If fMax Then Dec(rcCap.Right, fCaptionHeight);
      If fMin Then Dec(rcCap.Right, fCaptionHeight);
      If fClose Then Dec(rcCap.Right, fCaptionHeight);
    {$EndIf}

    Inc(rcCap.Right,1);
    Dec(rcCap.Bottom,1);
    {Draw caption background (don't use ExtTextOut, we may need dithered colors)}
    SelectObject(DC, GetStockObject(BLACK_PEN));
    Rectangle(DC, rcCap.Left, rcCap.Top, rcCap.Right , rcCap.Bottom);

    Pen:= CreatePen(PS_SOLID,1,ColorToRGB(clBtnHighlight));
    SelectObject(DC, Pen);
    MoveToEx(DC, rcCap.Left+2, rcCap.Top,NIL);
    LineTo(DC, rcCap.Left+2, rcCap.Bottom);
    MoveToEx(DC, rcCap.Left, rcCap.Top +1,NIL);
    LineTo(DC, rcCap.Right, rcCap.Top +1);

    hbrCap:= SelectObject(DC, hBrCap);   {swap brushes back again          }
    DeleteObject(hBrCap);                {delete the brush we created      }
    {Draw caption text here}
    TextLen := GetWindowTextLength(Handle);     {Get length of caption text}
    lpStr:= GlobalAllocPtr(GHND, TextLen + 2);  {Allocate a text buffer    }
    If lpStr<>NIL Then Begin
      GetWindowText(Handle, lpStr, TextLen + 1);{Copy in caption text      }
      rgbText:= SetTextColor(DC, rgbText);      {Swap text colors with DC  }
      {Prepare logical font structure to get a font to use.}
      FillChar(lFont, SizeOf(TLogFont), #0);    {Clear the font structure  }
      If Screen.PixelsPerInch > 96 Then
       lFont.lfHeight:= -(fCaptionHeight - 10)
      Else lFont.lfHeight:= -(fCaptionHeight - 6);
      lFont.lfCharSet:= ANSI_CHARSET;
      lFont.lfQuality:= DEFAULT_QUALITY;
      lFont.lfClipPrecision:= CLIP_LH_ANGLES OR CLIP_STROKE_PRECIS;
      If fCaptionHeight >= CaptionHeightThreshold Then
        lFont.lfWeight:= FW_BOLD;
      {Use small fonts for caption since it looks more like "System" than Arial}
      lFont.lfPitchAndFamily:= FF_SWISS;
      Font:= CreateFontIndirect(lFont);
      Font:= SelectObject(DC, Font);           {swap font with device context}
      {Calculate centering for caption text}
      GetTextExtentPoint(DC, lpStr, TextLen, Size);
      {$IfNdef HC_CAPTION_TEXT_HIGHLIGHT}
        {Shrink caption area to avoid overlapping control box & min/max buttons}
        If fSysMenu Then Inc(rcCap.Left, fCaptionHeight);
        If fMax Then Dec(rcCap.Right, fCaptionHeight);
        If fMin Then Dec(rcCap.Right, fCaptionHeight);
        If fClose Then Dec(rcCap.Right, fCaptionHeight);
      {$EndIf}
      cX:= rcCap.Left + ((rcCap.Right - rcCap.Left - Size.cX) DIV 2);
      cY:= rcCap.top + ((rcCap.Bottom - rcCap.Top - Size.cY) DIV 2);
      If rcCap.Left > cX Then cX:= rcCap.Left;
      {Draw caption text}
      ExtTextOut(DC, cX, cY, ETO_CLIPPED, @rcCap, lpStr, TextLen, NIL);
      Font:= SelectObject(DC, Font);                  {swap font back again}
      DeleteObject(Font);                             {delete it           }
      {Clean up device context & free memory}
      SetTextColor(DC, rgbText);                      {swap back text color}
      GlobalFreePtr(lpStr);                           {deallocate buffer   }
    End;
    {Draw control box, min button, and max button as needed}
    If fSysMenu Then DrawCaptionButton(DC, HTSYSMENU, False);
    If fMin Then DrawCaptionButton(DC, HTMINBUTTON, False);
    If fMax Then DrawCaptionButton(DC, HTMAXBUTTON, False);
    If fClose Then DrawCaptionButton(DC, HTCLOSE, False);
    ReleaseDC(Handle, DC);
    DeleteObject(Pen);
    Result:= True;
  end;
end;

function THCForm.DepressCaptionButton(Which: word): boolean;
var
  Rect: TRect;
  Msg: TMsg;
  Pressed : boolean;
begin
  Result:=False;
  Pressed:=True;
  If GetCaptionButtonRect(Which, Rect) Then Begin
    DrawCaptionButton(0, Which, Pressed);
    {Collect all mouse events until WM_LBUTTONUP}
    SetCapture(Handle);
    {Loop until the button is released}
    While TRUE Do Begin
      If PeekMessage(Msg, 0, WM_MOUSEFIRST, WM_MOUSELAST, PM_REMOVE) Then Begin
        Case Msg.Message Of
          WM_LBUTTONUP: Begin
            If Pressed Then Begin
              Pressed:= False;
              DrawCaptionButton(0, Which, Pressed);
            End;
            ReleaseCapture;
            Result:= PtInRect(Rect, Msg.pt);
            BREAK;
          End;
          WM_MOUSEMOVE: Begin
            If PtInRect(Rect, Msg.pt) Then Begin
              If NOT Pressed Then Begin
                Pressed:= True;
                DrawCaptionButton(0, Which, Pressed);
              End;
            End Else Begin
              If Pressed Then Begin
                Pressed:= False;
                DrawCaptionButton(0, Which, Pressed);
              End;
            End;
          End;
        End;
      End;
    End;
  End;
end;

function THCForm.DoSysMenu : boolean;
var
  DC: HDC;
  Rect: TRect;
  mPoint: TPoint;
  SysMenu : HMenu;
  wMove,wSize,wMinBox,wMaxBox,wRestore: word;
begin
  Result := False;  {Initially assume no menu}
  if TestWinStyle(WS_SYSMENU) then begin
    DC:= GetWindowDC(Handle);
    If DC <> 0 Then Begin
      {Invert the control box}
      DrawCaptionButton(DC, HTSYSMENU, True);
      {Pop up the mock-system menu}
      mPoint:= Point(0, -1);
      GetWindowRect(Handle, Rect);
      {Convert coordinates to screen coords. using functions in WinProcs unit}
      {("WinProcs" must be given to avoid calling TForm1's ClientToScreen() )}
      WinProcs.ClientToScreen(Handle, mPoint);
      WinProcs.ClientToScreen(Handle, Rect.BottomRight);
      SysMenu:= GetSystemMenu(Handle, False);

      {Initially assume all menu items should be grayed}
      wMove:= MF_GRAYED;
      wSize:= MF_GRAYED;
      wMinBox:= MF_GRAYED;
      wMaxBox:= MF_GRAYED;
      wRestore:= MF_GRAYED;
      {Now check the window styles, etc.}
      If NOT (IsIconic(Handle) OR IsZoomed(Handle)) Then Begin
        If TestWinStyle(WS_CAPTION) Then wMove:= MF_ENABLED;
        If TestWinStyle(WS_THICKFRAME) Then wSize:= MF_ENABLED;
      End;
      If TestWinStyle(WS_MINIMIZEBOX) Then wMinBox:= MF_ENABLED;
      If TestWinStyle(WS_MAXIMIZEBOX) OR IsIconic(Handle) Then wMaxBox:= MF_ENABLED;
      If IsZoomed(Handle) Then wRestore:= MF_ENABLED;
      EnableMenuItem(SysMenu, SC_MOVE,     wMove);
      EnableMenuItem(SysMenu, SC_SIZE,     wSize);
      EnableMenuItem(SysMenu, SC_MINIMIZE, wMinBox);
      EnableMenuItem(SysMenu, SC_MAXIMIZE, wMaxBox);
      EnableMenuItem(SysMenu, SC_RESTORE,  wRestore);

      TrackPopupMenu(SysMenu, 0, mPoint.X, mPoint.Y, 0, Handle, @Rect);
      DrawCaptionButton(DC, HTSYSMENU, False);
      ReleaseDC(Handle, DC);
    End;
    Result := True;
  End;
end;

procedure THCForm.WndProc(var Message : TMessage);
var
  DC: hDC;
  Pen: HPen;
  TextMetric: TTextMetric;
  PSysMenuString: PChar;
  ActSysMenuItem: integer;
  SysMenuItem: PSysMenuItem;
begin
  With Message Do Begin
    Case Msg Of
      WM_NCPAINT,
      WM_NCACTIVATE: Begin
        If HasCaption AND NOT IsIconic(Handle) Then Begin
          If Msg = WM_NCPAINT Then Begin
            fActiveCaption:= (Handle = GetActiveWindow);
          End Else
            fActiveCaption := (wParam <> 0);
          DrawCaption;
        End;
        inherited WndProc(Message);
      End;
      WM_MEASUREITEM: Begin
        {
          CtlType;      // type of control
          CtlID;        // combo box, list box, or button identifier
          itemID;       // menu item, variable-height list box, or combo box identifier
          itemWidth;    // width of menu item, in pixels
          itemHeight;   // height of single item in list box menu, in pixels
          itemData;     // application-defined 32-bit value
        }
        With PMeasureItemStruct(Message.LParam)^ Do Begin
          If Pointer(itemData)=Pointer(Self) Then Begin
            If CtlType=ODT_MENU Then Begin
              DC:= GetWindowDC(Handle);
              GetTextMetrics(DC,TextMetric);
              With TextMetric Do Begin
                itemWidth:=fSysMenuWidth*tmAveCharWidth+1;
                itemHeight:=tmHeight+1;
              End;
              ReleaseDC(Handle, DC);
            End;
          End;
        End;
        inherited WndProc(Message);
      End;
      WM_DRAWITEM: Begin
        {
          CtlType;
          CtlID;
          itemID;
          itemAction;
          itemState;
          hwndItem;
          hDC;
          rcItem;
          itemData;
        }
        With PDrawItemStruct(Message.LParam)^ Do Begin
          If Pointer(itemData)=Pointer(Self) Then Begin
            If CtlType=ODT_MENU Then Begin
              Case itemAction Of
                ODA_DRAWENTIRE: Begin
                  Pen:= CreatePen(PS_SOLID,1,ColorToRGB(clHCLightGreen));
                  SelectObject(hDC,Pen);
                  SetTextColor(hDC,ColorToRGB(clHCLightGreen));
                  SetBkColor(hDC, ColorToRGB(clHCDarkGreen{clHCMidGreen}));
                  PSysMenuString:=NIL;
                  If (fSysMenuItems<>NIL) AND (fSysMenuItems.Count>0) Then Begin
                    If fSysMenuItems.Search(@itemID,ActSysMenuItem) Then Begin
                      SysMenuItem:=fSysMenuItems.Items[ActSysMenuItem];
                      With SysMenuItem^ Do Begin
                        If SysMenuItemID=itemID Then Begin
                          PSysMenuString:=SysMenuItemText;
                        End;
                      End;
                    End;
                  End;
                  ExtTextOut(hDC, rcItem.Left, rcItem.Top, ETO_OPAQUE, @rcItem, NIL, 0, NIL);
                  DrawText(hDC,PSysMenuString, StrLen(PSysMenuString), rcItem, DT_SINGLELINE OR DT_EXPANDTABS OR DT_VCENTER);
                  DeleteObject(Pen);
                End;
                ODA_FOCUS: Begin
                End;
                ODA_SELECT: Begin
                End;
                Else Begin
                End;
              End;
            End;
          End;
        End;
        inherited WndProc(Message);
      End;
      Else Begin
        inherited WndProc(Message);
      End;
    End;
  End;
end;

{  }
{  WMNCxxxx NonClientRect messages }
{  }
procedure THCForm.WMNCCreate(var Message : TWMNCCreate);
var
  dwStyle : longInt;
begin
  fCaptionHeight:= (GetSystemMetrics(SM_CYCAPTION))+CaptionMinHeightPattern;
  dwStyle:= GetWindowLong(Handle, GWL_STYLE);
  If (dwStyle and WS_DLGFRAME) = WS_DLGFRAME Then Begin
    dwStyle:= dwStyle AND NOT longInt(WS_DLGFRAME);
    SetWindowLong(Handle, GWL_STYLE, dwStyle);
  End;

  fMenuUp:= False; {Indicate the system menu is not showing}
  fSysMenu:= TestWinStyle(WS_SYSMENU);
  fMin:= TestWinStyle(WS_MINIMIZEBOX);
  fMax:= TestWinStyle(WS_MAXIMIZEBOX);
  fClose:= TestWinStyle(WS_SYSMENU);

  inherited;  {Call default processing.}
end;

procedure THCForm.WMNCCalcSize(var Message : TWMNCCalcSize);
begin
  inherited;  {Call default processing.}
  If HasCaption and not IsIconic(Handle) Then Begin
    Inc(Message.CalcSize_Params^.RgRc[0].Top, fCaptionHeight);
    {If NOT Message.CalcValidRects Then Begin
    End;}
  End;
end;

procedure THCForm.WMNCHitTest(var Message : TWMNCHitTest);
var
  rcCap,rcMenu,rcMin,rcMax,rcClose: TRect;
  Point  : TPoint;
begin
  Inherited;  {Call default processing.}
  If (Message.Result = HTNOWHERE) AND HasCaption AND NOT IsIconic(Handle) Then Begin
    GetCaptionRect(rcCap);
    Point.X:=Message.Pos.X;
    Point.Y:=Message.Pos.Y;
    If PtInRect(rcCap, Point) Then Begin
      Message.Result:= HTCAPTION;

      GetCaptionButtonRect(HTSYSMENU,rcMenu);
      Point.X:=Message.Pos.X;
      Point.Y:=Message.Pos.Y;
      If PtInRect(rcMenu, Point) Then Begin
        Message.Result:= HTSYSMENU
      End Else Begin
        GetCaptionButtonRect(HTMINBUTTON,rcMin);
        Point.X:=Message.Pos.X;
        Point.Y:=Message.Pos.Y;
        If PtInRect(rcMin, Point) then Begin
          Message.Result:= HTMINBUTTON
        End Else Begin
          GetCaptionButtonRect(HTMAXBUTTON,rcMax);
          Point.X:=Message.Pos.X;
          Point.Y:=Message.Pos.Y;
          If PtInRect(rcMax, Point) Then Begin
            Message.Result:= HTMAXBUTTON;
          End Else Begin
            GetCaptionButtonRect(HTCLOSE,rcClose);
            Point.X:=Message.Pos.X;
            Point.Y:=Message.Pos.Y;
            If PtInRect(rcClose, Point) Then Begin
              Message.Result:= HTCLOSE;
            End;
          End;
        End;
      End;
    End;
  End;
  If Message.Result <> HTSYSMENU Then Begin
    fMenuUp := False;  {Indicate the system menu is not showing}
  End;
end;

procedure THCForm.WMNCLButtonDblClk(var Message : TWMNCLButtonDblClk);
begin
  If (Message.HitTest = HTSYSMENU) AND HasCaption AND NOT IsIconic(Handle) Then
    SendMessage(Handle, WM_CLOSE, 0, 0)
  Else inherited;  {Call default processing.}
end;

procedure THCForm.WMNCLButtonDown(var Message : TWMNCLButtonDown);
var
  mPoint: TPoint;
begin
  If HasCaption AND NOT IsIconic(Handle) Then Begin
    mPoint:= Point(Message.XCursor, Message.YCursor);
    Case Message.HitTest Of
      HTSYSMENU: Begin
        If NOT fMenuUp AND DoSysMenu Then
          fMenuUp := True
        Else fMenuUp := False;
      End;
      HTMINBUTTON: Begin
        If DepressCaptionButton(HTMINBUTTON) Then
          SendMessage(Handle, WM_SYSCOMMAND, SC_MINIMIZE, longInt(@mPoint));
      End;
      HTMAXBUTTON: Begin
        If DepressCaptionButton(HTMAXBUTTON) Then Begin
          If IsZoomed(Handle) Then
            SendMessage(Handle, WM_SYSCOMMAND, SC_RESTORE, longInt(@mPoint))
          Else SendMessage(Handle, WM_SYSCOMMAND, SC_MAXIMIZE, longInt(@mPoint));
        End;
      End;
      HTCLOSE: Begin
        If DepressCaptionButton(HTCLOSE) Then Begin
          SendMessage(Handle, WM_SYSCOMMAND, SC_CLOSE, longInt(@mPoint));
        End;
      End;
      Else Begin
        inherited;  {Call default processing.}
      End;
    End;
  End Else Begin
    inherited;  {Call default processing.}
  End;
end;

{  }
{  WMxxxx messages }
{  }
procedure THCForm.WMSize(var Message: TWMSize);
begin
  inherited;
  SetWindowRgn(Handle,0,True);
  fRGN:=CreateRoundRectRgn(0,0,Width,Height,fCaptionHeight*2,fCaptionHeight*2);
  If Message.SizeType<>SIZE_MAXIMIZED Then Begin
   SetWindowRgn(Handle,fRGN,True);
  End;
  DeleteObject(fRGN);
end;

procedure THCForm.WMSysChar(var Message : TWMSysChar);
begin
  If HasCaption AND (Message.CharCode = VK_SPACE) Then
    DoSysMenu
  Else inherited;  {Call default processing.}
end;

procedure THCForm.WMCommand(var Message : TMessage);
begin
  If Message.wParam >= $F000 Then
    PostMessage(Handle, WM_SYSCOMMAND, Message.wParam, Message.lParam);
  inherited;  {Call default processing.}
end;

procedure THCForm.WMKeyDown(var Message : TWMKeyDown);
var
  dwStyle : longInt;
begin
  dwStyle := GetWindowLong(Handle, GWL_STYLE);
  SetWindowLong(Handle, GWL_STYLE, dwStyle AND NOT longInt(WS_SYSMENU));
  inherited;  {Call default processing.}
  SetWindowLong(Handle, GWL_STYLE, dwStyle);
end;

procedure THCForm.WMKeyUp(var Message : TWMKeyUp);
var
  dwStyle : longInt;
begin
  dwStyle := GetWindowLong(Handle, GWL_STYLE);
  SetWindowLong(Handle, GWL_STYLE, dwStyle AND NOT longInt(WS_SYSMENU));
  inherited;  {Call default processing.}
  SetWindowLong(Handle, GWL_STYLE, dwStyle);
end;

procedure THCForm.WMSysKeyDown(var Message : TWMSysKeyDown);
var
  dwStyle : longInt;
begin
  dwStyle := GetWindowLong(Handle, GWL_STYLE);
  SetWindowLong(Handle, GWL_STYLE, dwStyle AND NOT longInt(WS_SYSMENU));
  inherited;  {Call default processing.}
  SetWindowLong(Handle, GWL_STYLE, dwStyle);
end;

procedure THCForm.WMSysKeyUp(var Message : TWMSysKeyUp);
var
  dwStyle : longInt;
begin
  dwStyle := GetWindowLong(Handle, GWL_STYLE);
  SetWindowLong(Handle, GWL_STYLE, dwStyle AND NOT longInt(WS_SYSMENU));
  inherited;  {Call default processing.}
  SetWindowLong(Handle, GWL_STYLE, dwStyle);
end;

procedure THCForm.WMGetMinMaxInfo(var Message : TWMGetMinMaxInfo);
var
  cX,cY: integer;
  rcMenu,rcMin,rcMax,rcClose: TRect;
begin
  If HasCaption and TestWinStyle(WS_THICKFRAME) Then Begin
    {The following functions return empty rects. if box/button doesn't exist}
    GetCaptionButtonRect(HTSYSMENU,rcMenu);
    GetCaptionButtonRect(HTMINBUTTON,rcMin);
    GetCaptionButtonRect(HTMAXBUTTON,rcMax);
    GetCaptionButtonRect(HTCLOSE,rcClose);

    cX := (rcMenu.Right - rcMenu.Left) +
          (rcMin.Right - rcMin.Left) +
          (rcMax.Right - rcMax.Left) +
          (rcClose.Right - rcClose.Left);
    cY := GetSystemMetrics(SM_CYFRAME);

    With Message.MinMaxInfo^.ptMinTrackSize do begin
      x := cX + 2 * fCaptionHeight;
      y := fCaptionHeight + 2 * cY - 1;
    End;
  End;
end;

constructor THCForm.Create(Owner: TComponent);
Var
  ActMenuStringPlace: packed array [0..256] of char;
  MenuItemsCount,ActMenuItem,ActItemLength: integer;
  SysMenu: HMenu;
  SysMenuItem: PSysMenuItem;
  ActMenuItemID: DWORD;
begin
  inherited Create(Owner);
  Color:= clHCMidGreen;
  fSysMenuItems:=TSortedDWORDKeyList.Create(False);
  fSysMenuWidth:=0;
  SysMenu:= GetSystemMenu(Handle, False);
  MenuItemsCount:=GetMenuItemCount(SysMenu);
  For ActMenuItem:=0 To MenuItemsCount Do Begin
    ActMenuItemID:=GetMenuItemID(SysMenu,ActMenuItem);
    GetMenuString(SysMenu,ActMenuItem,@ActMenuStringPlace,255,MF_BYPOSITION);
    ActItemLength:=StrLen(@ActMenuStringPlace);
    If ActItemLength>fSysMenuWidth Then Begin
      fSysMenuWidth:=ActItemLength;
    End;
    GetMem(SysMenuItem,SizeOf(TSysMenuItem));
    With SysMenuItem^ Do Begin
      SysMenuItemID:=ActMenuItemID;
      GetMem(SysMenuItemText,ActItemLength+1);
      StrCopy(SysMenuItemText,@ActMenuStringPlace);
    End;
    fSysMenuItems.Add( SysMenuItem );
    ModifyMenu(SysMenu,ActMenuItemID,MF_BYCOMMAND OR MF_OWNERDRAW,ActMenuItemID,Pointer(Self));
  End;

  SetWindowLong(Handle,GWL_EXSTYLE,GetWindowLong(Handle, GWL_EXSTYLE) AND (NOT WS_EX_STATICEDGE));

  fRGN:=CreateRoundRectRgn(0,0,Width,Height,fCaptionHeight*2,fCaptionHeight*2);
  SetWindowRgn(Handle,fRGN,False);
  DeleteObject(fRGN);
end;

destructor THCForm.Destroy;
Var
  SysMenuItem: PSysMenuItem;
  I: integer;
begin
  For I:=0 To fSysMenuItems.Count-1 Do Begin
    SysMenuItem:= fSysMenuItems.Items[I];
    With SysMenuItem^ Do Begin
      FreeMem(SysMenuItemText,StrLen(SysMenuItemText)+1);
    End;
    FreeMem(SysMenuItem,SizeOf(TSysMenuItem));
  End;
  fSysMenuItems.Free;
  inherited Destroy;
end;

end.
